@@ -2,6 +2,7 @@
 #define STRICT_R_HEADERS
 
 #include <Rinternals.h>
+#include <Rdefines.h>
 #include <R_ext/Rdynload.h>
 #include <librsvg/rsvg.h>
 #include <cairo.h>
@@ -10,6 +11,7 @@
 #include <cairo-ps.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdint.h>
 
 typedef struct {
   unsigned char *buf;
@@ -37,8 +39,9 @@ static void setup_render_handle(RsvgHandle *svg, cairo_t *cr){
 #if LIBRSVG_CHECK_VERSION(2,52,0)
   GError *err = NULL;
   RsvgRectangle viewport = {0.0};
-  //viewport.width = width;
-  //viewport.height = height;
+  cairo_surface_t *surface = cairo_get_target(cr);
+  viewport.width = cairo_image_surface_get_width(surface);
+  viewport.height = cairo_image_surface_get_height(surface);
   rsvg_handle_render_document(svg, cr, &viewport, &err);
   if(err != NULL)
     Rf_error("Failure in rsvg_handle_render_document: %s", err->message);
@@ -70,6 +73,43 @@ static SEXP write_bitmap(RsvgHandle *svg, int width, int height, double sx, doub
   return image;
 }
 
+static SEXP write_native_raster(RsvgHandle *svg, int width, int height, double sx, double sy){
+  cairo_surface_t *canvas = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+  cairo_t *cr = cairo_create(canvas);
+  cairo_scale(cr, sx, sy);
+  setup_render_handle(svg, cr);
+  cairo_surface_flush(canvas);
+
+  // Allocate an integer matrix of class 'nativeRaster'
+  SEXP image = PROTECT(Rf_allocVector(INTSXP, width*height));
+  SEXP dim = PROTECT(Rf_allocVector(INTSXP, 2));
+  INTEGER(dim)[0] = height;
+  INTEGER(dim)[1] = width;
+  Rf_setAttrib(image, R_DimSymbol, dim);
+  SET_CLASS(image, Rf_mkString("nativeRaster"));
+
+  // Get a pointer to the raw bytes from cairo in CAIRO_FORMAT_ARGB32
+  unsigned char *bp  = cairo_image_surface_get_data(canvas);
+  unsigned char *nrp = (unsigned char *)INTEGER(image);
+
+  // Note: this is not properly handling stride and is just assuming
+  // that the width of the canvas is width*channels
+  // Twiddle the colour channels
+  for (int i=0; i < height * width * 4; i+=4) {
+    nrp[i + 2] = bp[i + 0];
+    nrp[i + 1] = bp[i + 1];
+    nrp[i + 0] = bp[i + 2];
+    nrp[i + 3] = bp[i + 3];
+  }
+
+  // Free and return
+  UNPROTECT(2);
+  g_object_unref(svg);
+  cairo_surface_destroy(canvas);
+  cairo_destroy(cr);
+  return image;
+}
+
 static cairo_status_t write_func(void *ctx, const unsigned char *data, unsigned int length) {
   memory *mem = (memory*) ctx;
   #ifdef _WIN32
@@ -179,6 +219,11 @@ SEXP R_rsvg(SEXP data, SEXP rwidth, SEXP rheight, SEXP format, SEXP css){
     sx = sy = height / input_height;
     width = round(input_width * sx);
   }
+#if LIBRSVG_CHECK_VERSION(2,52,0)
+  /* New API seems to autoscale now */
+  sx = 1;
+  sy = 1;
+#endif
   switch(Rf_asInteger(format)){
   case 0:
     return write_bitmap(svg, width, height, sx, sy);
@@ -192,11 +237,13 @@ SEXP R_rsvg(SEXP data, SEXP rwidth, SEXP rheight, SEXP format, SEXP css){
     return write_stream(svg, width, height, sx, sy, cairo_ps_surface_create_for_stream, FALSE);
   case 5:
     return write_stream(svg, width, height, sx, sy, cairo_ps_surface_create_for_stream, TRUE);
+  case 6:
+    return write_native_raster(svg, width, height, sx, sy);
   }
   return R_NilValue;
 }
 
-SEXP R_librsvg_version(){
+SEXP R_librsvg_version(void){
   return Rf_mkString(LIBRSVG_VERSION);
 }
 
